----------------------------------------------------------------------------------
-- Company: 		 alexgiul@hotmai
-- Engineer: 		 Alessandro Giulianelli
-- 
-- Create Date:    22:14:43 01/13/2009 
-- Design Name: 
-- Module Name:    UDPReceiver - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity UDPReceiver is
    Port 
			( 
			clk : in  STD_LOGIC;
      rst : in  STD_LOGIC;
			  
			DEVICE_MAC: in STD_LOGIC_VECTOR (47 downto 0) ; -- fpga board mac 		
			IP : std_logic_vector (31 downto 0) ;           -- IP of this core																							 
			
			rd_en : in std_logic;								-- Request a memory read			  
			data_out : out std_logic_vector(7 downto 0);
			
			busy : out std_logic;									-- '1' during receiving IP frame
			empty : out std_logic;								-- queue empty
			full : out std_logic;									-- queue full
			data_count : out std_logic_vector(9 downto 0);--	elements into queue
		
			RX_DATA: in STD_LOGIC_VECTOR (3 downto 0);					-- 4 bit receive data line from the ethernet PHY
			RX_DV: in STD_LOGIC;													-- PHY receive data valid signal
			RX_CLK: in STD_LOGIC													-- PHY receiver clock
			);			  
end UDPReceiver;

architecture Behavioral of UDPReceiver is

component fifo_rx
	port (
	clk: IN std_logic;
	din: IN std_logic_VECTOR(7 downto 0);
	rd_en: IN std_logic;
	rst: IN std_logic;
	wr_en: IN std_logic;
	data_count: OUT std_logic_VECTOR(9 downto 0);
	dout: OUT std_logic_VECTOR(7 downto 0);
	empty: OUT std_logic;
	full: OUT std_logic);
end component;

-- Synplicity black box declaration
attribute syn_black_box : boolean;
attribute syn_black_box of fifo_rx: component is true;

	COMPONENT Ethernet
	PORT
		(
		clk : IN std_logic;
		rst : IN std_logic;
		DEVICE_MAC: in STD_LOGIC_VECTOR (47 downto 0) ; -- fpga board mac 		
		RX_DATA : IN std_logic_vector(3 downto 0);
		RX_DV : IN std_logic;
		RX_CLK : IN std_logic;          
		frameType : OUT std_logic;
		frameByte : OUT std_logic;
		frameData : OUT std_logic_vector(7 downto 0);
		newFrame : OUT std_logic;
		endFrame : OUT std_logic;
		frameValid : OUT std_logic
		);
	END COMPONENT;

type state_type is (idle,start_rcv,checkData,endTrasmission,start_ip_rcv,check_ip_rcv,
							payload_rcv,payload_cnt);
signal state, next_state: state_type ;

	
	SIGNAL frameType :  std_logic:= '0';
	SIGNAL frameByte :  std_logic:= '0';
	SIGNAL frameData :  std_logic_vector(7 downto 0):= (others => '0');
	SIGNAL din :  std_logic_vector(7 downto 0):= (others => '0'); -- data to queue
	
	
	SIGNAL newFrame,endFrame,memWriteReq :  std_logic := '0';
	
	signal counter : integer range 0 to 45 := 0;
	signal payloadCnt : unsigned (9 downto 0) :=(others => '0');
	signal payloadSize : std_logic_vector (9 downto 0) :=(others => '0');
	
	signal targetIP : unsigned (31 downto 0) :=(others => '0');
	
	

begin

inst_receiver_queue : fifo_rx
		port map (
			clk => clk,
			din => din,
			rd_en => rd_en,
			rst => rst,
			wr_en => memWriteReq,
			data_count => data_count,
			dout => data_out,
			empty => empty,
			full => full);
			
	Inst_Ethernet: Ethernet PORT MAP(
		clk => clk,
		rst => rst,
		DEVICE_MAC => DEVICE_MAC,
		RX_DATA => RX_DATA,
		RX_DV => RX_DV,
		RX_CLK => RX_CLK,
		frameType => frameType,
		frameByte => frameByte,
		frameData => frameData,
		newFrame => newFrame,
		endFrame => endFrame,
		frameValid => open
	);

process(state,newFrame,frameType,frameByte,frameData,counter,payloadCnt,RX_DV,endFrame)
begin

	case state is
		when idle =>			
			if newFrame = '1' and frameType = '1' then -- check if there is a new IP  frame
				next_state <= start_rcv;			
			else
				next_state <= idle;
			end if;

-------------------------
-- IP (UDP) receiver part
--------------------------		
		when start_rcv =>
		
			if frameByte = '1' then -- it's incoming a new byte...we see it
				next_state <= checkData;
			else
				next_state <= start_rcv;
			end if;
		
		when checkData =>
			if frameData = x"45" then -- start of ip packet
				next_state <= start_ip_rcv;
			else
				next_state <= checkData;
			end if;
		
		when start_ip_rcv =>
			if frameByte = '1' then -- it's incoming a new byte...we store it
				next_state <= check_ip_rcv;
			else
				next_state <= start_ip_rcv;
			end if;		
		
		when check_ip_rcv =>
			if endFrame = '1' or RX_DV = '0' then
				next_state <= endTrasmission;
			else
				next_state <= start_ip_rcv;
			end if;
		
			if counter = 8 then
				if framedata /= x"11" then -- if it is not UDP end...
					next_state <= endTrasmission;
				end if;
			elsif counter = 20 then
				if CONV_INTEGER(IP) /= CONV_INTEGER(targetIP) then -- the ip packet is not for me				
					next_state <= endTrasmission;
				end if;
			-- here is noc check for the udp port!
			elsif counter = 26 then
				next_state <= payload_rcv;	
			end if;
						
		when payload_rcv =>			

			if endFrame = '1' or RX_DV = '0' then
				next_state <= endTrasmission;
			elsif frameByte = '1' then -- it's incoming a new byte...we store it
				next_state <= payload_cnt;
			else
				next_state <= payload_rcv;
			end if;	
			
		when payload_cnt =>	
			
			if payloadCnt < conv_integer(payloadSize) then 
				next_state <= payload_rcv;				
			else
				next_state <= endTrasmission;
			end if;
-------------------------------
-------------------------------			
		when endTrasmission =>
				next_state <= idle;
			
		when others =>		
			next_state <= idle;
			
	end case;
end process;


process(clk,rst)
begin
	if rst = '1' then
		state <= idle;
	elsif clk'event and clk = '1' then
		state <= next_state;
	end if;
end process;

process(clk)
variable payload1 : std_logic_vector(9 downto 0);
variable payload2 : std_logic_vector(9 downto 0);
begin

if clk'event and clk = '1' then
	
	memWriteReq <= '0';
	
	case state is
	
		when idle =>
			payload1 := (others => '0');
			payload2 := (others => '0');
			busy <= '0';			
			din <= (others => '0');
									
			counter <= 0;
			payloadCnt <= (others => '0');

		when start_rcv =>			
			busy <= '1';
		when checkData =>		
		when start_ip_rcv =>		
		when check_ip_rcv =>			 
			  counter <= counter + 1;
			  
			  case counter is
					when 15 => targetIP(31 downto 24) <= unsigned(frameData);
					when 16 => targetIP(23 downto 16) <= unsigned(frameData);
					when 17 => targetIP(15 downto  8) <= unsigned(frameData);
					when 18 => targetIP(7 downto   0) <= unsigned(frameData);
					
					when 23 => payloadSize(9 downto 8) <= frameData(1 downto 0);
					when 24 => payloadSize(7 downto 0) <= frameData;
					when 25 => payloadSize <= payloadSize -9; -- 8 is the UDP header size
					when others =>			  
			  end case;

		when payload_rcv =>
				din <= frameData;
								
		when payload_cnt =>	
				payloadCnt <= payloadCnt + 1;
				memWriteReq <= '1';

		when endTrasmission =>
				payloadCnt <= (others => '0');
				
		when others =>			
		
	end case;		
end if;

end process;
end Behavioral;

