
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
-- synthesis translate_off
Library XilinxCoreLib;
-- synthesis translate_on


entity testerReceiver is
    Port 
				( 
				Pin_clk50 : in  STD_LOGIC;
				
				Pin_btn : in std_logic;
				Pin_rstBTN : in std_logic;
				
			  Pin_TX_CLK : IN std_logic;          		
				Pin_TX_EN : OUT std_logic;
				Pin_TX_DATA : buffer std_logic_vector(3 downto 0):= (others => '0');
	 	 
	 			Pin_RX_DATA :  in std_logic_vector(3 downto 0);
				Pin_RX_DV : in std_logic;
				Pin_RX_CLK :  in std_logic;
				Pin_LED : out std_logic_vector(7 downto 0)		  				
	 );
end testerReceiver;

architecture Behavioral of testerReceiver is

	COMPONENT UDPSender
	PORT(
		clk50 : IN std_logic;
		rst : IN std_logic;
		sendPacket : IN std_logic;
		SRC_IP : IN std_logic_vector(31 downto 0);
		DEVICE_MAC : IN std_logic_vector(47 downto 0);
		DST_IP : IN std_logic_vector(31 downto 0);
		DST_MAC : IN std_logic_vector(47 downto 0);
		
		read_data : out std_logic;
		data_in : IN std_logic_vector(7 downto 0);
		rdAddr:  out STD_LOGIC_VECTOR (9 downto 0);

		busy : OUT std_logic;
		
		TX_CLK : IN std_logic;          
		TX_EN : OUT std_logic;
		TX_DATA : buffer std_logic_vector(3 downto 0)
		);
	END COMPONENT;

	COMPONENT ram1k
	PORT(
		clka : IN std_logic;
		clkb : IN std_logic;
		wea : IN std_logic;
		addra : IN std_logic_vector(9 downto 0);
		addrb : IN std_logic_vector(9 downto 0);
		dia : IN std_logic_vector(7 downto 0);          
		dob : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;	

------------------------

	COMPONENT UDPReceiver
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;
		DEVICE_MAC : IN std_logic_vector(47 downto 0);
		IP : IN std_logic_vector(31 downto 0);
		rd_en : IN std_logic;
		RX_DATA : IN std_logic_vector(3 downto 0);
		RX_DV : IN std_logic;
		RX_CLK : IN std_logic;          
		data_out : OUT std_logic_vector(7 downto 0);
		busy : OUT std_logic;
		empty : OUT std_logic;
		full : OUT std_logic;
		data_count : OUT std_logic_vector(9 downto 0)
		);
	END COMPONENT;
	
-------- FSM ---------
type state_type is 
	(
	idle,receive,waitEndIPPacket,manageRXQueue,
	manageTXQueue,send,end_send,
	end_receive_ip,nextRXbyteInQueue
	);
signal state, next_state: state_type ;
----------------------

signal signal_rst : std_logic := '0';
signal data_count_rx:  std_logic_VECTOR(9 downto 0);

signal receiverBusy : std_logic := '0';								-- '1' while receiving packet
signal emptyReceiverQueue : std_logic :='0';

signal readEnable : std_logic :='0';
signal din,dout : std_logic_vector(7 downto 0):= (others => '0');

--------- RAM SIGNAL -----------
signal wraddress  : std_logic_vector(9 downto 0):= (others => '0');
signal rdaddress  : std_logic_vector(9 downto 0):= (others => '0');
signal acquiring : std_logic := '0';
------------ Sender Signals
signal sendPacket :  std_logic := '0';
signal busyReg : std_logic := '0';
-------------------------------

begin

----------sender part
-- my RAM (see XST documentation on how coding ram)
	Inst_ram1k: ram1k PORT MAP(
		clka => Pin_clk50,
		clkb => Pin_clk50,
		wea => acquiring,  -- write enable signal
		addra => wraddress,
		addrb => rdaddress,
		dia => din,--x"AA", -- data written into RAM
		dob => dout   -- data read from RAM
	);

-- UDP sender: fix packet lenght 1024 (see inside the core)
	Inst_UDPSender: UDPSender PORT MAP(
		clk50 => Pin_clk50,
		rst => signal_rst,
		sendPacket => sendPacket,
		SRC_IP => x"C0A8012C",				-- 192.168.1.44
		DEVICE_MAC => x"00aaaaaaaaaa",   -- fpga board mac 		
		DST_IP => x"C0A8022f", 				-- destination IP 192.168.2.47
		--DST_MAC => x"0015c5408862",		-- destination MAC mein xps1
		DST_MAC => x"0000b4c53f91",		-- destination MAC mein ACE Link usb-eth
		--DST_MAC => (others => '1'),		-- destination MAC 

		read_data => open,					-- read_data signal
		data_in => dout,--x"AA",	
		rdAddr => rdaddress,
		
		busy => busyReg,
		
		TX_CLK => Pin_TX_CLK,
		TX_EN => Pin_TX_EN,
		TX_DATA => Pin_TX_DATA
	);
	
-----------------------------------------
	Inst_UDPReceiver: UDPReceiver PORT MAP(
		clk => Pin_clk50,
		rst => signal_rst,
		DEVICE_MAC => x"00aaaaaaaaaa", 	-- fpga board mac 		
		IP => x"C0A8022C",							-- 192.168.2.44
		rd_en => readEnable,
		data_out => din,
		busy => receiverBusy,
		empty => emptyReceiverQueue,
		full => open,
		data_count => data_count_rx,
		RX_DATA => Pin_RX_DATA,
		RX_DV => Pin_RX_DV,
		RX_CLK => Pin_RX_CLK
	);
	
	process(state,Pin_btn,data_count_rx,receiverBusy,emptyReceiverQueue, busyReg)
	begin
	
	case state is
		when idle =>
			if Pin_btn = '1' then
				next_state <= receive;
			else
				next_state <= idle;
			end if;
			
		when receive =>
			if receiverBusy = '0' then --wait incoming packet
				next_state <= receive;
			else
				next_state <= waitEndIPPacket;
			end if;

		when waitEndIPPacket =>
			if receiverBusy = '1' then --wait end of incoming packet
				next_state <= waitEndIPPacket;
			else
				next_state <= end_receive_ip;
			end if;
		
		when end_receive_ip => -- dummy step 
			next_state<= manageRXqueue;
--------------------------------------
-- PAYLOAD is into Receiver queue, 
--------------------------------------
		when manageRXqueue =>
			next_state <= nextRXbyteInQueue;
		when nextRXbyteInQueue =>
			if emptyReceiverQueue = '1' then -- the RX queue is empty
				next_state <= manageTXQueue;
			else
				next_state <= manageRXqueue;
			end if;
--------------------------------------
-- End RX QUEUE manager
--------------------------------------			

--------------------------------------
-- PAYLOAD is into TX queue
--------------------------------------
		when manageTXQueue =>
			next_state <= send;
			
		when send =>
			if busyReg = '1' then
				next_state <= end_send;
			else 
				next_state <= send;
			end if;
		when end_send =>
			if busyReg = '1' then
				next_state <= end_send;
			else 
				next_state <= receive;			
			end if;
		when others =>
			next_state <= receive;
	end case;
end process;

process(Pin_clk50, Pin_rstBTN)
begin
	if Pin_rstBTN = '1' then
		state <= idle;
	elsif Pin_clk50'event and Pin_clk50 = '1' then
		state <= next_state;				
	end if;
end process;

process(Pin_clk50)
begin

	if Pin_clk50'event and Pin_clk50 = '1' then
			
			--- sender part
			acquiring <='0';
			sendPacket <= '0';
			sendPacket <= '0';
			----------------
			readEnable <= '0';
			signal_rst <= '0';
			Pin_LED <= x"00";			
			
		case state is		
			when idle =>
				signal_rst <= '1';		
-----sender part
wraddress <= (others => '0');	
---------------				
			when receive =>			
				Pin_LED <= x"01";
			when waitEndIPPacket =>
				Pin_LED <= x"02";
-- Now the packet is into the queue...so it can be processed			
			when manageRXQueue =>
				readEnable <= '1';										
			
			when nextRXbyteInQueue =>				
				acquiring <='1';
				wraddress <= wraddress + 1;
				Pin_LED <= x"04";
			when end_receive_ip =>

			when manageTXQueue =>							
			when send =>				
				sendPacket <= '1';
				Pin_LED <= x"FF";
			when end_send =>				
				Pin_LED <= x"FF";
				wraddress <= (others => '0');		
			when others =>
		end case;
	end if;
	
end process;
		

end Behavioral;

