----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    10:09:15 01/06/2009 
-- Design Name: 
-- Module Name:    UDPSender - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: UDP on port nr 1024
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;



entity UDPSender is
	
    Port ( clk50 : in  STD_LOGIC;										-- Any clock source = 50 Mhz
			  rst : in  STD_LOGIC;	
			  sendPacket : in  STD_LOGIC;           				-- Start send procedure of UDP packet

			  SRC_IP : in  STD_LOGIC_VECTOR (31 downto 0); 		-- the ip of the fpga board
			  DEVICE_MAC: in STD_LOGIC_VECTOR (47 downto 0); 	-- fpga board mac 		
			  DST_IP : in  STD_LOGIC_VECTOR (31 downto 0) ; 	 -- ip of the target computer
			  DST_MAC : in STD_LOGIC_VECTOR (47 downto 0); 
			  			  
			  read_data : out std_logic;								-- Request a memory read			  
			  data_in : in std_logic_vector(7 downto 0);
			  rdAddr:  out STD_LOGIC_VECTOR (9 downto 0);
			  
			  busy : out std_logic:= '0';									-- '1' during a trasmission @ 50 Mhz
			  
			  TX_CLK: in STD_LOGIC;										-- transmit clock from the PHY
           TX_EN: out STD_LOGIC:= '0';								-- transmit enable line to the PHY
           TX_DATA: buffer STD_LOGIC_VECTOR (3 downto 0) 	-- transmit data line to the PHY
	 );
end UDPSender;

architecture Behavioral of UDPSender is

component ethernetSnd is
    port (
        clk: in STD_LOGIC;										-- clock
        rst: in STD_LOGIC;										-- asynchronous active low reset
				DEVICE_MAC: in STD_LOGIC_VECTOR (47 downto 0); -- fpga board mac 		
        complete: in STD_LOGIC;									-- RAM operation complete signal
        rdData: in STD_LOGIC_VECTOR (7 downto 0);				-- read data bus from the RAM
        newFrame: in STD_LOGIC;									-- send a new frame signal
        frameSize: in STD_LOGIC_VECTOR (10 downto 0);			-- size of the frame to send
        destinationMAC: in STD_LOGIC_VECTOR (47 downto 0);		-- target MAC of the frame
        frameType: in STD_LOGIC;								-- type of the frame to send (ARP or IP)
        TX_CLK: in STD_LOGIC;									-- transmit clock from the PHY
        TX_EN: out STD_LOGIC;									-- transmit enable line to the PHY
        TX_DATA: buffer STD_LOGIC_VECTOR (3 downto 0);			-- transmit data line to the PHY
        rdRAM: out STD_LOGIC;									-- read RAM signal
        rdAddr: out STD_LOGIC_VECTOR (10 downto 0);				-- read address bus to the RAM
        frameSent: out STD_LOGIC      							-- frame sent signal to ARP
    );
end component;

	COMPONENT checksum_calc
	PORT(
		clk : IN std_logic;
		SRC_IP : in  STD_LOGIC_VECTOR (31 downto 0); -- the ip of the fpga board
		DST_IP : in  STD_LOGIC_VECTOR (31 downto 0);
		in16b : IN std_logic_vector(15 downto 0);
		newWord : IN std_logic;
		lastWord : IN std_logic;          
		result : OUT std_logic_vector(15 downto 0)
		);
	END COMPONENT;
	 	
type state_type is (idle,syncro,start,send1,send2,endTrasmission);
signal state, next_state: state_type ;

signal rd_en : std_logic := '0';
signal complete : std_logic :='0';
signal newFrame : std_logic;

signal frameSent : std_logic := '0';
signal rdRam : std_logic := '0';

signal myrdAddr: STD_LOGIC_VECTOR (10 downto 0):=(others =>'0');
signal myrdData : STD_LOGIC_VECTOR (7 downto 0):=(others =>'0'); -- data for ethernet

-- signal for checksum 
	SIGNAL wordReady :  std_logic := '0';   -- ready signal 
	SIGNAL lastWord:  std_logic := '0';
	SIGNAL in16bWord :  std_logic_vector(15 downto 0) := (others=>'0');
	SIGNAL result :  std_logic_vector(15 downto 0) := (others=>'0'); -- checksum calculated result

signal frameSize : std_logic_vector(10 downto 0) :=(others =>'0');
signal udpLenght : std_logic_vector(10 downto 0):=(others =>'0');

signal sendPacket1,sendPacket2 : std_logic := '0';
---------------- 
signal data_countBuffer: STD_LOGIC_VECTOR (10 downto 0):=(others =>'0');
begin

process(clk50)
begin
	if rising_edge(clk50) then
		sendPacket1 <= sendPacket;
		sendPacket2 <= sendPacket1;
	end if;
end process;	


	
-- UDPPacketSize : payload
-- 8 : UDP header
-- 20: IP Header
data_countBuffer <= "00" & CONV_STD_LOGIC_VECTOR(1022, 9);
--framesize <= (data_countBuffer+8+20+1);
--udpLenght <=  (data_countBuffer+8+1);
-- I want to send 1024 byte as payload
framesize <= CONV_STD_LOGIC_VECTOR(1052, framesize'length);
udpLenght <= CONV_STD_LOGIC_VECTOR(1032, udpLenght'length);


read_data <= rdRAM when conv_integer(myrdAddr(10 downto 0)) > 27 else '0';
rdAddr <= myrdAddr(9 downto 0)-28 when conv_integer(myrdAddr(10 downto 0)) > 27 else (others => '0');

ethernetCore: ethernetSnd 
    port map(
        clk => clk50,
        rst =>  rst,
		  DEVICE_MAC => DEVICE_MAC, 		 -- fpga board mac 		
        complete=> complete,									-- RAM operation complete signal (input) 		  
		  rdData => myrdData,
        newFrame=>newFrame,									-- send a new frame signal
        frameSize=> frameSize, 								-- size of the frame to send
        destinationMAC=> DST_MAC,					-- target MAC of the frame
        frameType=> '1',										-- type of the frame to send (ARP(0) or IP(1))
        TX_CLK=>TX_CLK,											-- transmit clock from the PHY
        TX_EN=>TX_EN,											-- transmit enable line to the PHY
        TX_DATA=>TX_DATA,										-- transmit data line to the PHY
        rdRAM=> rdRam,											-- read RAM signal
        rdAddr => myrdAddr,									-- read address bus to the RAM
        frameSent => frameSent								-- frame sent signal to ARP
    );

	
	checkSumCore: checksum_calc PORT MAP(
		clk => clk50,
		SRC_IP => SRC_IP,
		DST_IP => DST_IP,
		in16b => in16bWord,
		result => result,
		newWord => wordReady,
		lastWord => lastWord
	);


processFSM: process (clk50,rst)
begin
	if rst = '1' then
		state <= idle;
	elsif (clk50='1' and clk50'Event) then		
		state <= next_state;			
	end if;
	
end process processFSM;

-------------------------------------------------------------------------------
--	Check the current state
-------------------------------------------------------------------------------

FSMStateController : process (state, sendPacket2,rdRAM,frameSent)
begin

	case state is
		when idle => 				
			if sendPacket2 = '1' then							
				next_state <= syncro;
			else
				next_state <= idle;
			end if;
		
		when syncro =>
--			if data_countBuffer = data_countBuffer1 then
				next_state <= start;
	--		else
		--		next_state <= syncro;
		--	end if;
			
		
		when start =>
			next_state <= send1;
		
		when	send1 =>
		 if rdRam = '1' then
			next_state <= send2;
		 else
		   next_state <= send1;
		 end if;		 
		
			if frameSent = '1' then
					next_state <= endTrasmission;
			end if;
		
		when  send2 =>
			if frameSent = '1' then
				next_state <= endTrasmission;
			elsif rdRam = '0' then
				next_state <= send1;
			else
				next_state <= send2;
			end if;
						
		when endTrasmission =>						
				next_state <= idle;
		
		when others =>
			next_state <= idle;

	end case;

end process FSMStateController;

process3 : process (clk50)
begin
if (clk50='1' and clk50'Event) then
	
	case state is
		when idle =>
				busy <= '0';								
				newFrame <= '0';				
		when syncro =>
		
		when start =>
			busy <= '1';
			newFrame <= '1';						
			complete <='0';
			
		when send1 =>
			-- wait rdRAM signal from ethernetCore: this signal ask a new byte to send
			newFrame <= '0';
			complete <='0';
					
		when send2 =>
			-- data is ready, notifie it to ethernetCore
			complete <='1';
			
		when endTrasmission =>		
			complete <='0';
			busy <= '0';
		
		when others =>
			busy <= '0';
			complete <='0';			
		end case;	
	end if;	
end process process3;	

------------------------------------------------------------------
---- Memory buffer
-- the first part is the Header IP ...it's fixed or quite fixed
-- the second part connect the ethernet core to external memory
------------------------------------------------------------------
--
myMem : process (clk50)
variable step : std_logic := '0';
begin

if clk50'event and clk50 = '1' then
	
	wordReady <= '0';

-- These code lines make the memReadReq signal long one clock cycle 	instead of two as in the previous solution
	if rdRAM = '1' and step = '0' and conv_integer(myrdAddr(10 downto 0)) >= 27 then		
		rd_en <= '1';
		step := '1';
	elsif rdRAM = '1' and step = '1' then		
		rd_en <= '0';
		step := '0';
	else
		rd_en <= '0';
		step := '0';
	end if;

-- FIXED value for  IP

	if rdRAM = '1' then  -- IP packet	
			case conv_integer(myrdAddr(10 downto 0)) is
			-- IP header
				when 0  => myrdData <= x"45";
				when 1  => 
							myrdData <= x"00";
							in16bWord <= x"4500";
							wordReady <= '1';
				when 2  => myrdData <= x"0" & '0' & frameSize(10 downto 8);
				when 3  => 
							  myrdData <= frameSize(7 downto 0);
							  in16bWord <= x"0" & '0' & frameSize;
							  --in16bWord <= x"0" &  frameSize;
							  wordReady <= '1';
							  
				when 4  => myrdData <= x"B3";
				when 5  => myrdData <= x"FE";
								in16bWord <= x"B3FE";
								wordReady <= '1';
								
				when 6  => myrdData <= x"00";
				when 7  => myrdData <= x"00";
							  in16bWord <= x"0000";
							  wordReady <= '1';				
				when 8  => myrdData <= x"80";			
				when 9  => myrdData <= x"11"; -- IP protocol: 11 (=UDP)
							  lastWord <= '1';
							  in16bWord <= x"8011";
							  wordReady <= '1';
			-- IP checksum: 
				when 10  => myrdData <= result(15 downto 8);
				when 11  => 
								myrdData <= result(7 downto 0);
								lastWord <= '0';
				
	--# IP source 
				when 12  => myrdData <= std_logic_vector(SRC_IP(31 downto 24));
				when 13  => myrdData <= std_logic_vector(SRC_IP(23 downto 16));
				when 14  => myrdData <= std_logic_vector(SRC_IP(15 downto 8));
				when 15  => myrdData <= std_logic_vector(SRC_IP(7 downto 0));
	--# IP destination 
				when 16  => myrdData <= std_logic_vector(DST_IP(31 downto 24));
				when 17  => myrdData <= std_logic_vector(DST_IP(23 downto 16));
				when 18  => myrdData <= std_logic_vector(DST_IP(15 downto 8));
				when 19  => myrdData <= std_logic_vector(DST_IP(7 downto 0));
	
	--# UPD source port (1024): 04 00
--				when 20  => myrdData <= x"04";
--				when 21  => myrdData <= x"00";		
				when 20  => myrdData <= x"7d";	--0x7d02=32002
				when 21  => myrdData <= x"02";		
	--# UPD destination port (1024): 04 00
				when 22  => myrdData <= x"04";
				when 23  => myrdData <= x"00";		
	--# UDP payload length (18): 00 1A
				when 24  => myrdData <= x"0" & "0" & udpLenght(10 downto 8);
				when 25  => myrdData <= udpLenght(7 downto 0);
		
	--# UPD checksum: 
				when 26  => myrdData <= x"00";     
				when 27  => myrdData <= x"00";		
		
	-- Second part: we connect the ethernet core to external buffer where we store out payload
	----# UDP payload (18 bytes): 					
				when others => 
							myrdData <= data_in;
							--myrdData <= x"AA";
													
			end case;
		end if;
	end if;
end process;


end Behavioral;


