----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:28:05 01/06/2009 
-- Design Name: 
-- Module Name:    checksum_calc - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity checksum_calc is
    Port ( clk : in  STD_LOGIC;									-- any clock 
							 
			  SRC_IP : in  STD_LOGIC_VECTOR (31 downto 0);
			  DST_IP : in  STD_LOGIC_VECTOR (31 downto 0);
			  

			  in16b : in std_logic_vector(15 downto 0);		-- 16 bit wide word
			  result : out std_logic_vector(15 downto 0);	-- checksum		  
			  newWord : in std_logic;								-- signal to LOAD a new word
			  lastWord : in std_logic								-- signal to ask the result (high at least 2 clk cycles)
	 );
end checksum_calc;

architecture Behavioral of checksum_calc is

type state_type is (idle,load,valueOut);
signal state, next_state: state_type ;

begin
FSMStateController : process (state, newWord)
begin

	case state is
		when idle =>
			if newWord = '1' then			
				next_state <= load;
			else
				next_state <= idle;
			end if;

		when load => 										
				next_state <= valueOut;		

		when valueOut =>			
				next_state <= idle;					
				
		when others =>
			next_state <= idle;

	end case;

end process FSMStateController;

processFSM: process (clk)
begin	
	if (clk='1' and clk'Event) then		
			state <= next_state;				
	end if;	
end process processFSM;

process(clk)
variable  sum : unsigned(16 downto 0) := ((OTHERS => '0'));
begin

if clk'event and clk = '1' then

	case state is

		when idle =>	
			-- if sum = 0, we initialize this with constant value
		if sum = 0 then
			sum := unsigned(('0' & SRC_IP(31 downto 16)))+ unsigned(('0' & SRC_IP(15 downto 0)))+unsigned(('0' & DST_IP(31 downto 16)))+unsigned(('0' & DST_IP(15 downto 0)));
		end if;
		when load =>			
			sum := sum + unsigned(in16b);		
			IF (sum(16) = '1') THEN
				sum := '0' & sum(15 downto 0) + "1";
			END IF;

		when valueOut =>		
			IF (lastWord = '1') THEN
				FOR i IN 0 TO 15 LOOP
					result(i) <= NOT(std_logic(sum(i)));
					sum(i) := '0'; -- reset the variable
				END LOOP;
			END IF;		
		when others =>
	end case;


end if;	
end process;

end Behavioral;

